package com.faraphel.tasks_valider.connectivity.task.api

import com.faraphel.tasks_valider.connectivity.task.session.TaskPermission
import com.faraphel.tasks_valider.connectivity.task.session.TaskRole
import com.faraphel.tasks_valider.connectivity.task.session.TaskSession
import com.faraphel.tasks_valider.connectivity.task.session.TaskSessionManager
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD


/**
 * the HTTP API for the session manager
 */
class TaskSessionManagerApi(private val sessionManager: TaskSessionManager) {
    private val jsonParser = Gson()  ///< the json parser

    /**
     * Handle a HTTP Api request
     * @param taskSession the data of the client session
     * @param httpSession the data of the http session
     * @param path the path of the request
     */
    fun handleRequest(
        taskSession: TaskSession,
        httpSession: NanoHTTPD.IHTTPSession,
        path: MutableList<String>,
    ): NanoHTTPD.Response {
        // get the target session id
        val targetSessionId = path.removeFirstOrNull()

        return if (targetSessionId == null) {
            // no specific session targeted
            this.handleRequestGeneric(taskSession, httpSession)
        } else {
            // a specific session is targeted
            this.handleRequestSpecific(taskSession, httpSession, targetSessionId)
        }
    }

    /**
     * Handle a request with no specific session targeted
     */
    private fun handleRequestGeneric(
        taskSession: TaskSession,
        httpSession: NanoHTTPD.IHTTPSession,
    ): NanoHTTPD.Response {
        when (httpSession.method) {
            // get all the session data
            NanoHTTPD.Method.GET -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.READ))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                // return the session data
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "application/json",
                    jsonParser.toJson(taskSession)
                )
            }
            // other action are limited
            else -> {
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Unknown method"
                )
            }
        }
    }

    private fun handleRequestSpecific(
        taskSession: TaskSession,
        httpSession: NanoHTTPD.IHTTPSession,
        targetSessionId: String,
    ): NanoHTTPD.Response {
        when (httpSession.method) {
            // change a specific client session data
            NanoHTTPD.Method.POST -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.ADMIN))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "You are not allowed to update a session"
                    )

                // parse the content of the request
                val targetSession = jsonParser.fromJson(
                    httpSession.inputStream.bufferedReader().readText(),
                    TaskSession::class.java
                )

                // update the session
                this.sessionManager.setSessionData(targetSessionId, targetSession)

                // success message
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session updated"
                )
            }
            // delete the session
            NanoHTTPD.Method.DELETE -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.ADMIN))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "You are not allowed to delete a session"
                    )

                // delete the target session
                this.sessionManager.deleteSessionData(targetSessionId)

                // success message
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session deleted"
                )
            }
            // ignore other methods
            else -> {
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Invalid method"
                )
            }
        }
    }
}