package com.faraphel.tasks_valider.database.api

import com.faraphel.tasks_valider.connectivity.task.session.TaskPermission
import com.faraphel.tasks_valider.connectivity.task.session.TaskSession
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.api.entities.*
import com.faraphel.tasks_valider.database.api.entities.base.BaseApi
import com.faraphel.tasks_valider.database.entities.*
import fi.iki.elonen.NanoHTTPD

class TaskDatabaseApi(private val database: TaskDatabase) {
    private val api: Map<String, BaseApi> = mapOf(
        ClassEntity.TABLE_NAME to ClassApi(this.database.classDao()),
        PersonEntity.TABLE_NAME to PersonApi(this.database.personDao()),
        SessionEntity.TABLE_NAME to SessionApi(this.database.sessionDao()),
        SubjectEntity.TABLE_NAME to SubjectApi(this.database.subjectDao()),
        TaskEntity.TABLE_NAME to TaskApi(this.database.taskDao()),
        ValidationEntity.TABLE_NAME to ValidationApi(this.database.validationDao()),

        RelationClassPersonEntity.TABLE_NAME to RelationClassPersonApi(this.database.relationClassPersonDao()),
    )

    /**
     * handle an API request
     * @param taskSession the current user session
     * @param httpSession the http session
     * @param path the path of the request
     */
    fun handleRequest(
        taskSession: TaskSession,
        httpSession: NanoHTTPD.IHTTPSession,
        path: MutableList<String>
    ): NanoHTTPD.Response {
        // get the entity name
        val entityName = path.removeFirstOrNull()
            ?: return NanoHTTPD.newFixedLengthResponse(
                NanoHTTPD.Response.Status.BAD_REQUEST,
                "text/plain",
                "Missing entity name"
            )

        // get the correspond Api object for this entity
        val entityApi = this.api[entityName]
            ?: return NanoHTTPD.newFixedLengthResponse(
                NanoHTTPD.Response.Status.NOT_FOUND,
                "text/plain",
                "Unknown entity name"
            )

        // dispatch the request to the correct entity API
        when (httpSession.method) {
            // check if the data is in the database
            // TODO(Faraphel): should only be allowed to read data concerning the current class session
            NanoHTTPD.Method.HEAD -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.READ))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                return entityApi.head(httpSession)
            }
            // get the data from the database
            NanoHTTPD.Method.GET -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.READ))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                return entityApi.get(httpSession)
            }
            // insert the data into the database
            NanoHTTPD.Method.POST -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.WRITE))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                return entityApi.post(httpSession)
            }
            // delete the data from the database
            NanoHTTPD.Method.DELETE -> {
                // check the permission of the session
                if (taskSession.role.permissions.contains(TaskPermission.WRITE))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.FORBIDDEN,
                        "text/plain",
                        "Forbidden"
                    )

                return entityApi.delete(httpSession)
            }
            // other methods are not allowed
            else ->
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Method not allowed"
                )
        }
    }
}