package com.faraphel.tasks_valider.ui.screen.communication

import android.app.Activity
import android.widget.Toast
import androidx.compose.foundation.layout.Column
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.graphics.Color
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.bwf.BwfManager
import com.faraphel.tasks_valider.ui.screen.communication.internet.CommunicationInternetScreen
import com.faraphel.tasks_valider.ui.screen.communication.wifiP2p.CommunicationWifiP2pScreen


/**
 * CommunicationController is the main controller for the communication screen.
 * It is responsible for handling the navigation between the different communication methods.
 * It is also responsible for initializing the communication methods.
 * @param activity: The activity that hosts the communication screen.
 */
@Composable
fun CommunicationScreen(activity: Activity) {
    val controller = rememberNavController()

    NavHost(
        navController = controller,
        startDestination = "select"
    ) {
        composable("select") {
            CommunicationSelectContent(controller, activity)
        }
        composable("internet") {
            CommunicationInternetScreen(activity)
        }
        composable("wifi-p2p") {
            val bwfManager = BwfManager.fromActivity(activity)
            CommunicationWifiP2pScreen(activity, bwfManager)
        }
    }
}


/**
 * Communication screen that allows the user to choose the communication mode
 */
@Composable
fun CommunicationSelectContent(controller: NavController, activity: Activity) {
    val isWifiP2pSupported = BwfManager.isSupported(activity)

    Column {
        // internet communication mode
        Button(onClick = { controller.navigate("internet") }) {
            Text("Internet")
        }

        // wifi-direct communication mode
        Button(
            colors = ButtonDefaults.buttonColors(
                // if the WiFi-Direct is not supported, the button is grayed out
                containerColor = if (isWifiP2pSupported) Color.Unspecified else Color.Gray
            ),
            onClick = {
                // if the WiFi-Direct is supported, navigate to the WiFi-Direct screen
                if (isWifiP2pSupported) controller.navigate("wifi-p2p")
                // if the WiFi-Direct is not supported, show a toast message
                else Toast.makeText(activity, "WiFi-Direct is not supported on this device", Toast.LENGTH_SHORT).show()
            }
        ) {
            Text("WiFi-Direct")
        }
    }
}
